<?php

abstract class ChilexpressSoap extends SoapClient {

  /**
   * The namespace of header request.
   */
  const HEADER_NS = 'http://www.chilexpress.cl/common/HeaderRequest';



  /**
   * Header Variables
   */
  private $fechaHora = '?';
  private $idTransaccionNegocio = '?';
  private $idTransaccionOSB = '?';
  private $sistema = 'PHP';
  private $usuario = '?';
  private $oficinaCaja = '?';
  private $nodoHeader = '?';


  /**
   * Construct.
   */
  public function __construct(array $options)
  {
    $wsdl = $this->setupWsdl();

    parent::__construct($wsdl, $options);
  }


  /**
   * Helper function
   */
  protected function parameterParse(array $input, $arguments) {
    $parameters_object = new stdClass();
    foreach ($input as $key => $parameter) {
      if (!isset($arguments->{$key})) {
        $arguments->{$key} = $parameter;
      }
      if (is_array($parameter)) {
        $parameters_object->{$key} = $this->parameterParse($parameter, $arguments->{$key});
      }
      else {
        $parameters_object->{$key} = $arguments->{$key};
      }
    }

    return $parameters_object;
  }

  /**
   * Helper function
   */
  private function _arrayToObject(array $data) {
    $object = new stdClass();
    foreach ($data as $key => $value) {
      if (is_array($value)) {
        $object->{$key} = $this->_arrayToObject($value);
      }
      else {
        $object->{$key} = $value;
      }
    }

    return $object;
  }

  /**
   * Check the tree object and return the results.
   */
  private function mapping($map, $results) {
    $current = $results;
    if (!empty($map)) {
      foreach ($map as $key) {
        if (!isset($current->{$key})) {
          return NULL;
        }
        $current = $current->{$key};
      }
    }

    return $current;
  }

  /**
   * Execute the operation and mapping the results.
   */
  public function execute($operation, array $arguments = array()) {
    $arguments_object = $this->_arrayToObject($arguments);
    $input = $this->mappingInput();
    $output = $this->mappingOutput();


    $parameters = $this->parameterParse($input[$operation], $arguments_object);
    $results = $this->__soapCall($operation, array($parameters));

    return $this->mapping($output[$operation], $results);
  }

  /**
   * Override.
   */
  public function __soapCall($function_name, $arguments, $options = NULL, $input_headers = NULL, &$output_headers = NULL) {
    $this->updateDate();
    $this->updateHeaders();

    return parent::__soapCall($function_name, $arguments, $options, $input_headers, $output_headers);
  }

  /**
   * Get the current date.
   */
  private function updateDate() {
    $date = new DateTime('NOW');

    $this->fechaHora = $date->format(DateTime::ATOM);
  }

  /**
   * Update the headers with the new values.
   */
  private function updateHeaders() {
    $transaccion = new SoapVar(
      array(
        $this->soapVar('fechaHora'),
        $this->soapVar('idTransaccionNegocio'),
        $this->soapVar('idTransaccionOSB'),
        $this->soapVar('sistema'),
        $this->soapVar('usuario'),
        $this->soapVar('oficinaCaja'),
        $this->soapVar('nodoHeader'),
      ),
      SOAP_ENC_OBJECT, null, null, 'transaccion', self::HEADER_NS
    );

    $_transaccion = new SoapVar(array($transaccion), SOAP_ENC_OBJECT, null, null, 'transaccion', self::HEADER_NS);
    $header = new SoapHeader($this->getHeaderNS(), 'headerRequest', $_transaccion);

    $this->__setSoapHeaders($header);
  }

  private function soapVar($parameter) {
    $value = $this->{$parameter};

    return new SoapVar($value, XSD_STRING, null, null, $parameter, self::HEADER_NS);
  }
}
