<?php

/**
 * @file
 * This file has the administrative pages for this module.
 */


/**
 * Get a form for settings.
 *
 * @param mixed $form
 *   form element
 * @param mixed $form_state
 *   current form state
 *
 * @return array
 *   form element
 */
function webpay_settings_admin($form, &$form_state) {
  $form['webpay_debug'] = array(
    '#type' => 'checkbox',
    '#title' => t('Debug'),
    '#description' => t('If enabled, the data sent and received from Webpay will be logged in the watchdog. Use this only for certification.'),
    '#default_value' => variable_get('webpay_debug', FALSE),
  );

  return system_settings_form($form);
}


/**
 * A list of commerce code.
 */
function webpay_admin_configs_list() {

  $header = array('ID', t('Commerce Code'), t('Environment'), t('Created'), t('Operation'));
  $rows = array();
  $environments = webpay_get_environment();
  $configs = webpay_get_config();
  foreach ($configs as $config) {
    $operations = array(
      l(t('Edit'), 'admin/config/webpay/commerces/' . $config->id),
      l(t('Test'), 'admin/config/webpay/commerces/' . $config->id . '/test'),
      l(t('Delete'), 'admin/config/webpay/commerces/' . $config->id . '/delete'),
    );
    $rows[] = array(
      $config->id,
      $config->commerce_code,
      $environments[$config->environment],
      format_date($config->created),
      implode(' ', $operations),
    );
  }

  $build['table'] = array(
    '#theme' => 'table',
    '#rows' => $rows,
    '#header' => $header,
    '#empty' => t('No configs'),
  );

  return $build;
}


/**
 *
 * @param type $form
 * @param type $form_state
 * @param type $config
 */
function webpay_admin_config_form($form, &$form_state, $config = NULL) {

  if (is_object($config)) {
    $form['webpay_config_id'] = array(
      '#type' => 'hidden',
      '#value' => $config->id,
    );
  }

  $form['commerce_code'] = array(
    '#type' => 'textfield',
    '#maxlength' => 12,
    '#size' => 12,
    '#title' => t('Commerce Code'),
    '#description' => t('The commerce code hiven from transbank team.'),
    '#default_value' => isset($config->commerce_code) ? $config->commerce_code : '',
    '#required' => TRUE,
  );

  $form['environment'] = array(
    '#type' => 'select',
    '#title' => t('Environment'),
    '#options' => webpay_get_environment(),
    '#default_value' => isset($config->environment) ? $config->environment : NULL,
    '#required' => TRUE,
  );

  $form['commerce_keys'] = array(
    '#type' => 'container',
    '#tree' => TRUE,
  );

  $form['commerce_keys']['private_key'] = array(
    '#type' => 'textarea',
    '#title' => t('Private key'),
    '#required' => TRUE,
    '#default_value' => isset($config->commerce_keys['private_key']) ? $config->commerce_keys['private_key'] : '',
  );
  $form['commerce_keys']['public_key'] = array(
    '#type' => 'textarea',
    '#title' => t('Public key'),
    '#required' => TRUE,
    '#default_value' => isset($config->commerce_keys['public_key']) ? $config->commerce_keys['public_key'] : '',
  );
  $form['commerce_keys']['webpay_key'] = array(
    '#type' => 'textarea',
    '#title' => t('Webpay key'),
    '#required' => TRUE,
    '#default_value' => isset($config->commerce_keys['webpay_key']) ? $config->commerce_keys['webpay_key'] : '',
  );

  $form['actions'] = array(
    '#type' => 'actions',
  );
  $form['actions']['save'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  $form['actions']['save_test'] = array(
    '#type' => 'submit',
    '#value' => t('Save and Test'),
  );

  return $form;
}


function webpay_admin_config_form_submit($form, &$form_state) {
  $trigger = $form_state['triggering_element'];

  $config = array();
  $values = $form_state['values'];
  if (isset($values['webpay_config_id'])) {
    $config['id'] = $values['webpay_config_id'];
  }

  $config['commerce_code'] = $values['commerce_code'];
  $config['environment'] = $values['environment'];
  $config['commerce_keys'] = $values['commerce_keys'];

  $id = webpay_config_save($config);
  drupal_set_message(t('The commerce %code was saved', array('%code' => $config['commerce_code'])));
  $form_state['redirect'] = 'admin/config/webpay/commerces';
  if ($trigger['#value'] == t('Save and Test')) {
    $form_state['redirect'] = 'admin/config/webpay/commerces/' . $id . '/test';
  }

}


function webpay_admin_config_delete_form($form, &$form_state, $config) {
  $form['id'] = array(
    '#type' => 'hidden',
    '#value' => $config->id,
  );

  return confirm_form($form,
    t('Are you sure to delete the config !commerce_code?', array('!commerce_code' => $config->commerce_code)),
    'admin/config/webpay/commerces');
}


function webpay_admin_config_delete_form_submit($form, &$form_state) {
  $id = $form_state['values']['id'];
  webpay_config_delete($id);
  drupal_set_message(t('A commerce was deleted'));
  $form_state['redirect'] = 'admin/config/webpay/commerces';
}


function webpay_admin_config_test_form($form, &$form_state, $config) {
  $end = filter_input(INPUT_GET, 'end');
  $token = filter_input(INPUT_POST, 'token_ws');

  $form['information'] = array(
    '#type' => 'container'
  );

  if ($end && $token && ($transaction = webpay_get_transaction_by_token($token))) {
    // Check if the transaction exists.
    $form_state['step'] = 'end';

    $form['information']['transaction'] = array(
      '#markup' => '<p>' . t('The transaction was successful.') . '</p>',
    );
    $form['information']['voucher'] = array(
      '#theme' => 'webpay_voucher',
      '#transaction' => $transaction,
    );
    $form['init'] = array(
      '#type' => 'submit',
      '#value' => t('Try again'),
      '#submit' => array('webpay_admin_config_test_form_init_submit'),
    );
  }

  if (empty($form_state['step'])) {
    $form_state['step'] = 'init';

    $form['information']['introduction'] = array(
      '#markup' => '<p>' . t('This section can help you to test the connection with Webpay. Press "Test connection" to start.') . '</p>',
    );

    $form['init'] = array(
      '#type' => 'submit',
      '#value' => t('Test connection'),
      '#submit' => array('webpay_admin_config_test_form_init_submit'),
    );
  }

  if ($form_state['step'] == 'go') {

    $amount = $form_state['data_test']['amount'];
    $order_number = $form_state['data_test']['order_number'];
    $options = array(
      'absolute' => TRUE,
      'query' => array('end' => 'end'),
    );
    $finalUrl = url('admin/config/webpay/commerces/' . $config->id . '/test', $options);

    $webpayService = new WebpayNormalService($config, 'WebpayTest');
    $response = $webpayService->initTransaction($order_number, $amount, $finalUrl, 'testTransaction');

    if (!$response) {
      $form['information']['problem'] = array(
        '#markup' => '<p>' . t('Exists some problems with the connection with Webpay. Check the keys.') . '</p>',
      );
    }
    else {
      $form['information']['success'] = array(
        '#markup' => '<p>' . t('The site successfully connected to Webpay. Now you can do the transaction. Press "Go to Webpay"') . '</p>',
      );
      $form['information']['data'] = array(
        '#theme' => 'item_list',
        '#items' => array(
          t('Amount') . ': $' . number_format($amount, 0),
          t('Order Number') . ': ' . $order_number,
        ),
      );
      $form['#action'] = $response->url;
      $form['token_ws'] = array(
        '#type' => 'hidden',
        '#value' => $response->token,
      );
      $form['go'] = array(
        '#type' => 'submit',
        '#value' => t('Go to Webpay'),
      );
    }
  }

  return $form;
}


/**
 * A submit handler.
 */
function webpay_admin_config_test_form_init_submit($form, &$form_state) {
  $form_state['step'] = 'go';
  $form_state['rebuild'] = TRUE;

  $form_state['data_test'] = array(
    'amount' => rand(10000, 150000),
    'order_number' => rand(1, 50000),
  );
}

/**
 * Helper function to storage the currencies supported by Webpay.
 *
 * @return array
 *   List of currencies and they corresponded code.
 */
function webpay_get_currencies() {
  return array_keys(webpay_admin_default_code());
}


/**
 * Return a table with all logs related to Webpay's KCC.
 *
 * @return string
 *   HTML table with pager
 */
function webpay_admin_log() {
  $transaction = webpay_get_all_webpay_transaction(TRUE);
  $output = t('No log recorded');
  if (!empty($transaction)) {
    foreach ($transaction[0] as $key => $value) {
      $header[] = $key;
    }
    foreach ($transaction as $value) {
      $rows[] = $value;
    }
    $output = theme('table', array('rows' => $rows, 'header' => $header));
    $output .= theme('pager', array());
  }
  return $output;
}

