<?php

/**
 * @file
 * This file contains the primary pages of this module.
 */


/**
 * The return page.
 *
 * @param string $instance_id
 *  A referencial instance for other modules.
 * @param object $config
 *  The config object of the commerce code.
 *
 * @return mixed
 */
function webpay_return_page($instance_id, $config) {
  $token = filter_input(INPUT_POST, 'token_ws');
  if (!isset($token)) {
    drupal_access_denied();
    drupal_exit();
  }

  $webpayService = new WebpayNormalService($config);
  $response = $webpayService->getTransactionResult($token);

  if (!$response) {
    return t('There is an error with the connection with Webpay.');
  }

  // We going to save the transaction.
  $transaction = entity_create('webpay_transaction', array(
    'config_id' => $config->id,
    'commerce_code' => $config->commerce_code,
    'token' => $token,
    'order_number' => $response->buyOrder,
    'session_id' => $response->sessionId,
    'transaction_date' => strtotime($response->transactionDate),
    'vci' => $response->VCI,
    'card_number' => $response->cardDetail->cardNumber,
    'authorization_code' => $response->detailOutput->authorizationCode,
    'payment_type_code' => $response->detailOutput->paymentTypeCode,
    'response_code' => $response->detailOutput->responseCode,
    'amount' => $response->detailOutput->amount,
    'shares_number' => $response->detailOutput->sharesNumber,
  ));

  try {
    $transaction->save();
  } catch (Exception $err) {
    $watchdog_message = 'Failed to save the transaction in the database: @err';
    $watchdog_variable = array('@err' => $err->getMessage());
    watchdog('WEBPAY', $watchdog_message, $watchdog_variable, WATCHDOG_ERROR);

    return t('Some error detected');
  }


  if (($transaction->vci == "TSY" || $transaction->vci == "") && $transaction->response_code === 0) {
    module_invoke_all('webpay_transaction_accepted', $instance_id, $config, $transaction);

    $path = drupal_get_path('module', 'webpay');
    drupal_add_css($path . '/css/webpay-return.css');
    drupal_add_js($path . '/js/webpayReturn.js');
    $build = array(
      '#theme' => 'webpay_return',
      '#token' => $token,
      '#url' => $response->urlRedirection,
    );
    print drupal_render($build);

    return;
  }
  // Was rejected
  module_invoke_all('webpay_transaction_rejected', $instance_id, $config, $transaction);

  drupal_set_title(t('Order failed'));
  return theme('webpay_failure', array(
    'order_id' => $transaction->order_number,
  ));
}
